VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsLVExcelExport"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'==============================================================================
'Encapsulates the export of a ListView to Excel.
'
'Scope:     DLL-public creatable.
'==============================================================================


Option Explicit

Dim MyListView As ListView 'ListView that should be exported

'Export destination filename, title, highlight selected entries
Dim mFilename As String, mHeader As String, mMark As Boolean

'draw borders and backgrounds, only export selected entries
Dim mBordersAndBackground As Boolean, mOnlySelected As Boolean

'converts a Visual Basic OLE Windows-Color (&H80xxxxxx) to the original color.
Private Declare Function TranslateColor Lib "olepro32.dll" Alias "OleTranslateColor" (ByVal clr As OLE_COLOR, ByVal palet As Long, col As Long) As Long

'Sets the ListView that should be exported.
Property Let ListView(ListView As Object)
    Set MyListView = ListView
End Property

'==============================================================================
'Reason:    Displays the dialog for selecting a export-destination (file).
'
'Returns:   True if the dialog has been confirmed, otherwise False.
'==============================================================================
Function ShowFileDialog() As Boolean
    With New frmCDialogDummy
        With .cdialog
            .Flags = cdlOFNOverwritePrompt
            .Filter = "Mircrosoft Excel-Workbook (*.xls)|*.xls|All files (*.*)|*.*"
            .DialogTitle = "Select export-destination"
            .DefaultExt = "*.xls"
            .Filename = mFilename
            On Error GoTo erro
            .ShowSave
            mFilename = .Filename
            ShowFileDialog = True
erro:
        End With
    End With
End Function

'==============================================================================
'Reason:    Determines the currently installed Excel-Version.
'
'Returns:   The Excel-Version as String, or "(not found)" if an error has
'           occured during determining the version.
'==============================================================================
Property Get ExcelVersion() As String
    On Error GoTo erro
    Dim e As Object
    Set e = CreateObject("Excel.Application")
    ExcelVersion = e.Version
    Set e = Nothing
    Exit Property
    
erro:
    ExcelVersion = "(not found)"
End Property

'The following properties are used to get or set the export-attributes.

Property Get Filename() As String
    Filename = mFilename
End Property

Property Let Filename(ByVal value As String)
    mFilename = value
End Property

Property Get Header() As String
    Header = mHeader
End Property

Property Let Header(ByVal value As String)
    mHeader = value
End Property

Property Get Mark() As Boolean
    Mark = mMark
End Property

Property Let Mark(ByVal value As Boolean)
    mMark = value
End Property

Property Get BordersAndBackground() As Boolean
    BordersAndBackground = mBordersAndBackground
End Property

Property Let BordersAndBackground(ByVal value As Boolean)
    mBordersAndBackground = value
End Property

Property Get OnlySelected() As Boolean
    OnlySelected = mOnlySelected
End Property

Property Let OnlySelected(ByVal value As Boolean)
    mOnlySelected = value
End Property

'converts a Visual Basic OLE Windows-Color (&H80xxxxxx) to the original color.
Private Function GetRealColor(ByVal Color As OLE_COLOR) As Long
     TranslateColor Color, 0, GetRealColor
End Function

'==============================================================================
'Reason:    Exports the given ListView with the given attributes with VBA to
'           Excel.
'
'Returns:   True if the operation was successful, otherwise False.
'==============================================================================
Function ExportListViewToExcel() As Boolean
    On Error GoTo erro
    Dim c As Object, R As Object, i As Long, j As Long
    
    'Needed constants
    Const xlEdgeLeft = 7
    Const xlEdgeBottom = 9
    Const xlHAlignCenter = &HFFFFEFF4
    Const xlHAlignLeft = &HFFFFEFDD
    Const xlHAlignRight = &HFFFFEFC8
    Const xlContinuous = 1
    Const xlColorIndexAutomatic = &HFFFFEFF7
    Const xlDash = &HFFFFEFED
    Const xlHairline = 1
    Const xlMedium = &HFFFFEFD6
    
    
   
    With CreateObject("Excel.Application") 'Load Excel
        With .Workbooks.Add 'Open new workbook
            .Activate
            Set c = .Worksheets.Add 'Create new worksheet
            With c
                .Activate
                
                'set the cell-font to the font of the ListView
                With .Cells.Font
                    .Bold = MyListView.Font.Bold
                    .Italic = MyListView.Font.Italic
                    .Name = MyListView.Font.Name
                    .Size = MyListView.Font.Size
                    .Strikethrough = MyListView.Font.Strikethrough
                End With
                
                
                Dim fmtObj As Object, writeCount As Long
                'export the entries
                For i = 1 To MyListView.ListItems.Count
                    'If that item should be exported
                    If (OnlySelected And MyListView.ListItems(i).Selected) Or Not OnlySelected Then
                        writeCount = writeCount + 1
                        
                        'iterate all columns
                        For j = 1 To MyListView.ColumnHeaders.Count
                            Set R = .Cells(writeCount + 1, j) 'select destination-cell
                            With R
                                If j = 1 Then 'use the item itself if its the first column
                                    Set fmtObj = MyListView.ListItems(i)
                                Else 'use the respective ListSubItem for all other columns
                                    On Error Resume Next
                                    Set fmtObj = MyListView.ListItems(i).ListSubItems(j - 1)
                                    If Err.Number <> 0 Then
                                        Set fmtObj = Nothing
                                        Err.Clear
                                    End If
                                    On Error GoTo erro
                                End If
                                
                                If Not fmtObj Is Nothing Then
                                    .value = fmtObj.Text 'set text
                                        
                                    .NumberFormat = "@" 'set number format (@ = Text)
                                    
                                    'set forecolor and bold-attribute corresponding to the current column
                                    .Font.Color = GetRealColor(fmtObj.ForeColor)
                                    .Font.Bold = fmtObj.Bold
                                End If
                                
                                'if borders and backgrounds should be exported
                                If BordersAndBackground Then
                                    'highlight item if needed
                                    If Mark And MyListView.ListItems(i).Selected Then
                                        .Interior.Color = GetRealColor(vbHighlight)
                                        .Font.Color = GetRealColor(vbHighlightText)
                                    End If
                                    'display bottom-border
                                    .Borders.Item(xlEdgeBottom).ColorIndex = xlColorIndexAutomatic
                                    .Borders.Item(xlEdgeBottom).LineStyle = xlDash
                                    .Borders.Item(xlEdgeBottom).Weight = xlHairline
                                End If
                            End With
                        Next j
                    End If
                Next i
                
                
                'write headers:
                For i = 1 To MyListView.ColumnHeaders.Count
                    Set R = .Cells(1, i) 'select destination-cell
                    With R
                        .value = MyListView.ColumnHeaders(i).Text 'set text
                        .NumberFormat = "@" 'set number format (@ = Text)
                        'calculate column-width and set it
                        .ColumnWidth = MyListView.ColumnHeaders(i).Width / Screen.TwipsPerPixelX / 7.37
                        
                        
                        'if border and backgrounds should be drawn...
                        If BordersAndBackground Then
                            '... set cell-background to gray and borders left and bottom
                            .Interior.Color = &HC0C0C0
                            .Borders.Item(xlEdgeLeft).Color = vbWhite
                            .Borders.Item(xlEdgeLeft).LineStyle = xlContinuous
                            .Borders.Item(xlEdgeBottom).ColorIndex = xlColorIndexAutomatic
                            .Borders.Item(xlEdgeBottom).LineStyle = xlContinuous
                        End If
                    End With
                    'select whole column...
                    Set R = .Range(.Cells(1, i), .Cells(writeCount + 1, i))
                    With R
                        '... and set the text-align
                        .HorizontalAlignment = Switch( _
                            MyListView.ColumnHeaders(i).Alignment = lvwColumnLeft, xlHAlignLeft, _
                            MyListView.ColumnHeaders(i).Alignment = lvwColumnCenter, xlHAlignCenter, _
                            MyListView.ColumnHeaders(i).Alignment = lvwColumnRight, xlHAlignRight)
                    End With
                Next i
                'select all header-cells
                Set R = .Range(.Cells(1, 1), .Cells(1, MyListView.ColumnHeaders.Count))
                With R
                    'format headers
                    .Font.Bold = True
                    .Font.Size = .Font.Size + 2
                End With
                
                
                'Set a border around the whole ListView-area
                If BordersAndBackground Then
                    Set R = .Range(.Cells(1, 1), .Cells(writeCount + 1, MyListView.ColumnHeaders.Count))
                    With R
                        .BorderAround xlContinuous, xlMedium, xlColorIndexAutomatic
                    End With
                End If
                
                'set page header and footer
                With .PageSetup
                    .LeftHeader = Header
                    .RightHeader = "&D, &T"
                    .CenterFooter = "Seite &P von &N"
                End With
            End With
            
            
            On Error Resume Next
            Kill Filename 'delete existing file if needed
            On Error GoTo erro
            
            'create file
            .SaveAs Filename, , , , , , , , False
        End With
        .Quit 'close Excel
    End With
    ExportListViewToExcel = True
    Exit Function
    
erro:
    MsgBox "Error " & Err.Number & " while exporting to Excel occured:" & vbCrLf & vbCrLf & Err.Description, vbCritical, "Export list"
End Function

Private Sub Class_Initialize()
    mBordersAndBackground = True
End Sub
