VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsLVExcelExport"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'==============================================================================
'Kapselt den Export eines ListView nach Excel ab.
'
'Scope:     DLL-ffentlich erstellbar.
'==============================================================================


Option Explicit

Dim MyListView As ListView 'Zu exportierendes ListView

'Export-Ziel-Dateiname, Titel, Markieren von ausgewhlten Eintrgen
Dim mFilename As String, mHeader As String, mMark As Boolean

'Rahmen und Hintergrnde zeichnen, Nur ausgewhlte Eintrge exportieren
Dim mBordersAndBackground As Boolean, mOnlySelected As Boolean

'Wandelt eine Visual-Basic-OLE-Windows-Farbe (&H80xxxxxx) in die Originalfarbe um.
Private Declare Function TranslateColor Lib "olepro32.dll" Alias "OleTranslateColor" (ByVal clr As OLE_COLOR, ByVal palet As Long, col As Long) As Long

'Setzt das zu exportierende ListView.
Property Let ListView(ListView As Object)
    Set MyListView = ListView
End Property

'==============================================================================
'Zweck:     Zeigt den Dialog zum Auswhlen eines Exportziels (Datei) an.
'
'Rckgabe:  True, falls der Dialog besttigt wurde, sonst False.
'==============================================================================
Function ShowFileDialog() As Boolean
    With New frmCDialogDummy
        With .cdialog
            .Flags = cdlOFNOverwritePrompt
            .Filter = "Mircrosoft Excel-Arbeitsmappe (*.xls)|*.xls|Alle Dateien (*.*)|*.*"
            .DialogTitle = "Exportziel auswhlen"
            .DefaultExt = "*.xls"
            .Filename = mFilename
            On Error GoTo erro
            .ShowSave
            mFilename = .Filename
            ShowFileDialog = True
erro:
        End With
    End With
End Function

'==============================================================================
'Zweck:     Ermittelt die installierte Excel-Version.
'
'Rckgabe:  Die Excel-Version als String, oder "(nicht gefunden)", falls
'           ein Fehler beim Ermitteln der Version aufgetreten ist.
'==============================================================================
Property Get ExcelVersion() As String
    On Error GoTo erro
    Dim e As Object
    Set e = CreateObject("Excel.Application")
    ExcelVersion = e.Version
    Set e = Nothing
    Exit Property
    
erro:
    ExcelVersion = "(nicht gefunden)"
End Property

'Die folgenden Eigenschaften ermglichen das Auslesen bzw. Setzen der
'Export-Attribute.

Property Get Filename() As String
    Filename = mFilename
End Property

Property Let Filename(ByVal value As String)
    mFilename = value
End Property

Property Get Header() As String
    Header = mHeader
End Property

Property Let Header(ByVal value As String)
    mHeader = value
End Property

Property Get Mark() As Boolean
    Mark = mMark
End Property

Property Let Mark(ByVal value As Boolean)
    mMark = value
End Property

Property Get BordersAndBackground() As Boolean
    BordersAndBackground = mBordersAndBackground
End Property

Property Let BordersAndBackground(ByVal value As Boolean)
    mBordersAndBackground = value
End Property

Property Get OnlySelected() As Boolean
    OnlySelected = mOnlySelected
End Property

Property Let OnlySelected(ByVal value As Boolean)
    mOnlySelected = value
End Property

'Wandelt eine Visual-Basic-OLE-Windows-Farbe (&H80xxxxxx) in die Originalfarbe um.
Private Function GetRealColor(ByVal Color As OLE_COLOR) As Long
     TranslateColor Color, 0, GetRealColor
End Function

'==============================================================================
'Zweck:     Exportiert das zurvor angegebenen ListView mit den zuvor
'           angegebenen Attributen per VBA nach Excel.
'
'Rckgabe:  True, falls der Vorgang erfolgreich war, sonst False.
'==============================================================================
Function ExportListViewToExcel() As Boolean
    On Error GoTo erro
    Dim c As Object, R As Object, i As Long, j As Long
    
    'Bentigte Konstanten
    Const xlEdgeLeft = 7
    Const xlEdgeBottom = 9
    Const xlHAlignCenter = &HFFFFEFF4
    Const xlHAlignLeft = &HFFFFEFDD
    Const xlHAlignRight = &HFFFFEFC8
    Const xlContinuous = 1
    Const xlColorIndexAutomatic = &HFFFFEFF7
    Const xlDash = &HFFFFEFED
    Const xlHairline = 1
    Const xlMedium = &HFFFFEFD6
    
    
   
    With CreateObject("Excel.Application") 'Excel laden
        With .Workbooks.Add 'neue Arbeitsmappe ffnen.
            .Activate
            Set c = .Worksheets.Add 'neues Arbeitsblatt erstellen
            With c
                .Activate
                
                'Zellenschriftart auf die Schriftart des ListViews setzen.
                With .Cells.Font
                    .Bold = MyListView.Font.Bold
                    .Italic = MyListView.Font.Italic
                    .Name = MyListView.Font.Name
                    .Size = MyListView.Font.Size
                    .Strikethrough = MyListView.Font.Strikethrough
                End With
                
                
                Dim fmtObj As Object, writeCount As Long
                'Alle Eintrge exportieren
                For i = 1 To MyListView.ListItems.Count
                    'Falls dieses Item ausgegeben werden soll.
                    If (OnlySelected And MyListView.ListItems(i).Selected) Or Not OnlySelected Then
                        writeCount = writeCount + 1
                        
                        'Alle Spalten durchlaufen
                        For j = 1 To MyListView.ColumnHeaders.Count
                            Set R = .Cells(writeCount + 1, j) 'Zielzelle auswhlen
                            With R
                                If j = 1 Then 'Bei erster Spalte das Item selbst verwenden
                                    Set fmtObj = MyListView.ListItems(i)
                                Else 'Bei anderen Spalten das jeweilige ListSubItem verwenden.
                                    On Error Resume Next
                                    Set fmtObj = MyListView.ListItems(i).ListSubItems(j - 1)
                                    If Err.Number <> 0 Then
                                        Set fmtObj = Nothing
                                        Err.Clear
                                    End If
                                    On Error GoTo erro
                                End If
                                
                                If Not fmtObj Is Nothing Then
                                    .value = fmtObj.Text 'Text setzen
                                        
                                    .NumberFormat = "@" 'Nummernformat setzen (@ = Text)
                                    
                                    'Fordergrundfarbe und Fett-Attribut entsprechend der aktuellen Spalte
                                    'setzen.
                                    .Font.Color = GetRealColor(fmtObj.ForeColor)
                                    .Font.Bold = fmtObj.Bold
                                End If
                                
                                'Falls Rahmen und Hintergrnde exportiert werden sollen
                                If BordersAndBackground Then
                                    'ggf. Item als markiert darstellen
                                    If Mark And MyListView.ListItems(i).Selected Then
                                        .Interior.Color = GetRealColor(vbHighlight)
                                        .Font.Color = GetRealColor(vbHighlightText)
                                    End If
                                    'Rahmen unten zeichnen
                                    .Borders.Item(xlEdgeBottom).ColorIndex = xlColorIndexAutomatic
                                    .Borders.Item(xlEdgeBottom).LineStyle = xlDash
                                    .Borders.Item(xlEdgeBottom).Weight = xlHairline
                                End If
                            End With
                        Next j
                    End If
                Next i
                
                
                'Header schreiben:
                For i = 1 To MyListView.ColumnHeaders.Count
                    Set R = .Cells(1, i) 'Zielzelle auswhlen
                    With R
                        .value = MyListView.ColumnHeaders(i).Text 'Text setzen
                        .NumberFormat = "@" 'Nummernformat setzen (@ = Text)
                        'Spaltenbreite berechnen und setzen
                        .ColumnWidth = MyListView.ColumnHeaders(i).Width / Screen.TwipsPerPixelX / 7.37
                        
                        
                        'Falls Rahmen und Hintergrnde gezeichnet werden sollen...
                        If BordersAndBackground Then
                            '... Zelle grau hinterlegen, weie Trennlinie und Rahmen
                            'unten setzen.
                            .Interior.Color = &HC0C0C0
                            .Borders.Item(xlEdgeLeft).Color = vbWhite
                            .Borders.Item(xlEdgeLeft).LineStyle = xlContinuous
                            .Borders.Item(xlEdgeBottom).ColorIndex = xlColorIndexAutomatic
                            .Borders.Item(xlEdgeBottom).LineStyle = xlContinuous
                        End If
                    End With
                    'Gesamten Spalte auswhlen...
                    Set R = .Range(.Cells(1, i), .Cells(writeCount + 1, i))
                    With R
                        '... und Textausrichtung festlegen.
                        .HorizontalAlignment = Switch( _
                            MyListView.ColumnHeaders(i).Alignment = lvwColumnLeft, xlHAlignLeft, _
                            MyListView.ColumnHeaders(i).Alignment = lvwColumnCenter, xlHAlignCenter, _
                            MyListView.ColumnHeaders(i).Alignment = lvwColumnRight, xlHAlignRight)
                    End With
                Next i
                'Alle Header-Zellen auswhlen.
                Set R = .Range(.Cells(1, 1), .Cells(1, MyListView.ColumnHeaders.Count))
                With R
                    'Header formatieren
                    .Font.Bold = True
                    .Font.Size = .Font.Size + 2
                End With
                
                
                'Rahmen um den gesamten ListView-Bereich.
                If BordersAndBackground Then
                    Set R = .Range(.Cells(1, 1), .Cells(writeCount + 1, MyListView.ColumnHeaders.Count))
                    With R
                        .BorderAround xlContinuous, xlMedium, xlColorIndexAutomatic
                    End With
                End If
                
                'Seiten-Kopf- und -Fuzeile setzen.
                With .PageSetup
                    .LeftHeader = Header
                    .RightHeader = "&D, &T"
                    .CenterFooter = "Seite &P von &N"
                End With
            End With
            
            
            On Error Resume Next
            Kill Filename 'Bestehende Datei lschen falls vorhanden
            On Error GoTo erro
            
            'Datei erstellen
            .SaveAs Filename, , , , , , , , False
        End With
        .Quit 'Excel schlieen
    End With
    ExportListViewToExcel = True
    Exit Function
    
erro:
    MsgBox "Fehler " & Err.Number & " beim Exportieren nach Excel aufgetreten:" & vbCrLf & vbCrLf & Err.Description, vbCritical, "Liste exportieren"
End Function

Private Sub Class_Initialize()
    mBordersAndBackground = True
End Sub
